unit MailMerg;

{

+------------------------------------------------------------------------------+
| TMailMerge Component Version 1.0                                             |
+------------------------------------------------------------------------------+
| Author: Herv Roz                                                            |
| e-mail: hroz@chez.com                                                        |
+------------------------------------------------------------------------------+
| Description:                                                                 |
|   This component encapsulates the merge fonction of Microsoft Word. It use   |
| the OLE Automation to do that.                                               |
| The data source can be either a ODBC data source name or the table of a      |
| Microsoft Access database.                                                   |
+------------------------------------------------------------------------------+
| Properties:                                                                  |
|   DatabaseName: The name of the ODBC data source or the name of the          |
|     MS Access database (*.MDB).                                              |
|   Directory(*): Defauft directory only for the ODBC data source.             |
|   TableName: The file name of the ODBC data source (*.TXT,*.DBF,*.DB,...) or |
|     the table name of the MS Access database.                                |
|   DocumentName: The document name (*.DOC,*.DOT).                             |
|   Destination: Mail Merge Destination (See the MS Word documentation).       |
|   DocumentType: The document format (See the MS Word documentation).         |
|   SuppressBlankLines: Set to True if you want to suppress blank lines when   |
|     merge a empty field.                                                     |
|   FirstRecord: The number of the first record to merge.                      |
|   LastRecord: The number of the last record to merge.                        |
|   TableType(*): The table type only for the ODBC data source                 |
|   Pause: Set to True if you want that MS Word stops each time a merge error  |
|     occurs.                                                                  |
|                                                                              |
| (*): Read only property                                                      |
+------------------------------------------------------------------------------+
| Methods:                                                                     |
|   procedure Execute: Performs the specified merge file operation.            |
+------------------------------------------------------------------------------+
| Events:                                                                      |
+------------------------------------------------------------------------------+

+------------------------------------------------------------------------------+
| Installation                                                                 |
+------------------------------------------------------------------------------+
| 1. Unzip the file to any directory you like, for instance                    |
|   C:\Delphi 3\Source\MailMerg, ...                                           |
| 2. Start Delphi and close any projects and/or files                          |
| 3. From the menu, choose 'Components/Install Components...'                  |
| 4. In the dialog, click 'Browse'                                             |
| 5. Navigate to your chosen directory and select 'MailMerg.pas'               |
| 6. Click OK the way back                                                     |
| 7. In the dialog you have now set the unit file                              |
| 8. Click OK to perform the install                                           |
+------------------------------------------------------------------------------+

+------------------------------------------------------------------------------+
| ******************************* IMPORTANT ********************************** |
+------------------------------------------------------------------------------+
|     This software is absolutely free. Use it and enjoy Delphi.               |
|     All I want is to know why and how you are using this                     |
|     software.                                                                |
|                                                                              |
|     Suggestions for improvement and fixes are always welcome,                |
|     although no guarantee is made when we'll implemement them.               |
|                                                                              |
|     If you do use this component, please drop me an email                    |
|     telling me how you used it.                                              |
|                                                                              |
|     For bug reports, suggestions, new versions etc. contact me:              |
|                                                                              |
|     e-mail: hroz@chez.com                                                    |
|     URL: http://www.chez.com/hroz                                            |
|                                                                              |
+------------------------------------------------------------------------------+

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  comobj, DBTables, registry, dsgnintf;

{$I msword8.inc}

type
  TMailMergeDestination = (
    mdSendToNewDocument, mdSendToPrinter, mdSendToEmail, mdSendToFax);

  TMailMergeDocumentType = (
    dtNotAMergeDocument, dtFormLetters, dtMailingLabels, dtEnvelopes, dtCatalog);

  TMailMerge = class(TComponent)
  private
    { Dclarations prives }
    FDatabaseName: string;
    FDestination: TMailMergeDestination;
    FDocumentName: TFileName;
    FDocumentType: TMailMergeDocumentType;
    FFirstRecord: Integer;
    FLastRecord: Integer;
    FPause: Boolean;
    FSuppressBlankLines: Boolean;
    FTableName: TFileName;

    function GetDirectory: string;
    function GetTableType: string;
  protected
    { Dclarations protges }
    MSWord: OLEVariant;

    procedure DoExecute;
  public
    { Dclarations publiques }
    procedure Execute;
  published
    { Dclarations publies }
    property DatabaseName: string read FDatabaseName write FDatabaseName;
    property Destination: TMailMergeDestination read FDestination
      write FDestination default mdSendToNewDocument;
    property Directory: string read GetDirectory;
    property DocumentName: TFileName read FDocumentName write FDocumentName;
    property DocumentType: TMailMergeDocumentType read FDocumentType
      write FDocumentType default dtFormLetters;
    property FirstRecord: Integer read FFirstRecord write FFirstRecord
      default wdDefaultFirstRecord;
    property LastRecord: Integer read FLastRecord write FLastRecord
      default wdDefaultLastRecord;
    property Pause: Boolean read FPause write FPause;
    property SuppressBlankLines: Boolean read FSuppressBlankLines write FSuppressBlankLines;
    property TableName: TFileName read FTableName write FTableName;
    property TableType: string read GetTableType;
  end;

  TDocumentNameProperty = class(TPropertyEditor)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TTableNameProperty = class(TPropertyEditor)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TDatabaseNameProperty = class(TPropertyEditor)
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure register;

implementation

{ TMailMerge }

function TMailMerge.GetDirectory: string;
var
  Registry: TRegistry;
begin
  if DatabaseName <> '' then
  begin
    Registry:=TRegistry.Create;
    try
      with Registry do
      begin
        if OpenKey('Software\ODBC\ODBC.INI\'+DatabaseName,False) then
        begin
          Result:=ReadString('DefaultDir');
          CloseKey;
        end
        else Result:='';
      end;
    finally
      Registry.Free;
    end;
  end
  else Result:='';
end;

function TMailMerge.GetTableType: string;
var
  Registry: TRegistry;
begin
  if DatabaseName <> '' then
  begin
    Registry:=TRegistry.Create;
    try
      with Registry do
      begin
        if OpenKey('Software\ODBC\ODBC.INI\'+DatabaseName,False) then
        begin
          Result:=ReadString('FIL');
          CloseKey;
        end
        else Result:='';
      end;
    finally
      Registry.Free;
    end;
  end
  else Result:='';
end;

procedure TMailMerge.Execute;

  procedure CheckProperty(PropertyName: string; FileName: TFileName);
  begin
    if FileName = '' then
      raise Exception.Create('Proprit '+PropertyName+' manquante.');
    if not FileExists(FileName) then
      raise Exception.Create('Le fichier n''existe pas'+#13+
        'Fichier ou rpertoire inexistant.'+#13+'Fichier: '+
        ExpandFileName(FileName));
  end;

begin
  CheckProperty('DocumentName', FDocumentName);
  if Session.IsAlias(DatabaseName) then
  begin
    if FDatabaseName = '' then
      raise Exception.Create('Proprit ''DatabaseName'' manquante.');
    CheckProperty('TableName', FTableName);
  end
  else CheckProperty('DatabaseName', FDatabaseName);
  DoExecute;
end;

procedure TMailMerge.DoExecute;
const
  MainDocumentType: array [TMailMergeDocumentType] of Longint = (
    wdNotAMergeDocument, wdFormLetters, wdMailingLabels, wdEnvelopes, wdCatalog);

  MailMergeToWhere: array [TMailMergeDestination] of Longint = (
    wdSendToNewDocument, wdSendToPrinter, wdSendToEmail, wdSendToFax);

  procedure GetMSWordObject(AppName: string);
  begin
    try
      try
        MSWord:=GetActiveOLEObject(AppName);
      except
        MSWord:=CreateOLEObject(AppName);
      end;
      MSWord.Visible:=True;
    except on EOleException do
      MSWord:=Null;
    end;
    Application.ProcessMessages;
  end;

var
  MainDocument: Variant;
  WordMailMerge: Variant;
  Connection,SQLStatement: string;
  FDirectory, TempDatabaseName: string;
begin
  try
    GetMSWordObject('Word.Application');
    MSWord.Documents.Open(FileName:=DocumentName);
    Application.ProcessMessages;

    MainDocument:=MSWord.ActiveDocument;

    WordMailMerge:=MainDocument.MailMerge;
    WordMailMerge.MainDocumentType:=MainDocumentType[FDocumentType];

    if Session.IsAlias(DatabaseName) then
    begin
      TempDatabaseName:='';
      FDirectory:=GetDirectory;
      Connection:=Format('DSN=%s;DBQ=%s;DefaultDir=%s;FIL=%s;',
        [FDatabaseName, FDirectory, FDirectory, GetTableType]);
      SQLStatement:=Format('SELECT * FROM `%s`', [ExtractFileName(FTableName)]);
    end
    else
    begin
      TempDatabaseName:=DatabaseName;
      Connection:=Format('TABLE %s', [FTableName]);
      SQLStatement:=Format('SELECT * FROM [%s]', [FTableName]);
    end;
    WordMailMerge.OpenDataSource(Name:=TempDatabaseName,
      ConfirmConversions:=False, ReadOnly:=False, LinkToSource:=True,
      AddToRecentFiles:=False, PasswordDocument:='', PasswordTemplate:='',
      WritePasswordDocument:='', WritePasswordTemplate:='', Revert:=False,
      Format:=wdOpenFormatAuto, Connection:=Connection,
      SQLStatement:=SQLStatement, SQLStatement1:='');

    WordMailMerge.Destination:=MailMergeToWhere[FDestination];
    WordMailMerge.MailAsAttachment:=False;
    WordMailMerge.MailAddressFieldName:='';
    WordMailMerge.MailSubject:='';
    WordMailMerge.SuppressBlankLines:=FSuppressBlankLines;
    WordMailMerge.DataSource.FirstRecord:=wdDefaultFirstRecord;
    WordMailMerge.DataSource.LastRecord:=wdDefaultLastRecord;
    if (FFirstRecord <> 0) and (FLastRecord >= FFirstRecord) then
    begin
      WordMailMerge.DataSource.FirstRecord:=FFirstRecord;
      WordMailMerge.DataSource.LastRecord:=FLastRecord;
    end;

    WordMailMerge.Execute(Pause:=FPause);

    Application.ProcessMessages;

  finally
    MSWord := Unassigned;
  end;
end;

{ TDocumentNameProperty }

function TDocumentNameProperty.GetValue: string;
begin
  Result:=(GetComponent(0) as TMailMerge).DocumentName;
end;

procedure TDocumentNameProperty.SetValue(const Value: string);
begin
  (GetComponent(0) as TMailMerge).DocumentName:=Value;
  Modified;
end;

function TDocumentNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paMultiSelect];
end;

procedure TDocumentNameProperty.Edit;
var
  Dialog: TOpenDialog;
begin
  Dialog := TOpenDialog.Create(nil);
  try
    with Dialog do
    begin
      DefaultExt := 'doc';
      Filter := 'Documents Word (*.doc)|*.DOC|Modles de documents (*.dot)|*.DOT';
      FileName:=(GetComponent(0) as TMailMerge).DocumentName;
      if Execute then
        with (GetComponent(0) as TMailMerge) do
        begin
          DocumentName:=FileName;
          Modified;
        end;
    end;
  finally
    Dialog.Free;
  end;
end;

{ TTableNameProperty }

function TTableNameProperty.GetValue: string;
begin
  Result:=(GetComponent(0) as TMailMerge).TableName;
end;

procedure TTableNameProperty.SetValue(const Value: string);
begin
  (GetComponent(0) as TMailMerge).TableName:=Value;
  Modified;
end;

function TTableNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paMultiSelect];
end;

procedure TTableNameProperty.Edit;
var
  Dialog: TOpenDialog;
begin
  Dialog:=TOpenDialog.Create(nil);
  try
    with Dialog do
    begin
      DefaultExt:='txt';
      Filter:='Fichiers texte (*.txt)|*.TXT';
      FileName:=(GetComponent(0) as TMailMerge).TableName;
      if Execute then
        with (GetComponent(0) as TMailMerge) do
        begin
          TableName:=FileName;
          Modified;
        end;
    end;
  finally
    Dialog.Free;
  end;
end;

{ TDatabaseNameProperty }

function TDatabaseNameProperty.GetValue: string;
begin
  Result:=(GetComponent(0) as TMailMerge).DatabaseName;
end;

procedure TDatabaseNameProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values:=TStringList.Create;
  try
    Session.GetAliasNames(Values);
    for I:=0 to Values.Count-1 do Proc(Values.Strings[I]);
  finally
    Values.Free;
  end;
end;

procedure TDatabaseNameProperty.SetValue(const Value: string);
begin
  (GetComponent(0) as TMailMerge).DatabaseName:=Value;
  Modified;
end;

function TDatabaseNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result:=[paMultiSelect, paValueList, paSortList, paRevertable];
end;

procedure register;
begin
  RegisterComponents('VCL', [TMailMerge]);
  RegisterPropertyEditor(TypeInfo(string), TMailMerge, 'DatabaseName', TDatabaseNameProperty);
  RegisterPropertyEditor(TypeInfo(string), TMailMerge, 'DocumentName', TDocumentNameProperty);
  RegisterPropertyEditor(TypeInfo(string), TMailMerge, 'TableName', TTableNameProperty);
end;

end.
